import os

import pytest

from impala._thrift_api import ImpalaHttpClient


@pytest.fixture()
def proxy_env():
    reset_value = os.environ.get("HTTPS_PROXY")
    os.environ["HTTPS_PROXY"] = "https://foo:%3F%40%3D@localhost"
    yield "proxy_env"
    if reset_value is None:
        del os.environ["HTTPS_PROXY"]
    else:
        os.environ["HTTPS_PROXY"] = reset_value


class TestHttpTransport(object):
    def test_proxy_auth_header(self, proxy_env):
        client = ImpalaHttpClient(
            uri_or_host="https://localhost:443/cliservice",
        )
        assert client.proxy_auth == "Basic Zm9vOj9APQ=="



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec


from thrift.transport import TTransport
all_structs = []


class fb_status(object):
    """
    Common status reporting mechanism across all services

    """
    DEAD = 0
    STARTING = 1
    ALIVE = 2
    STOPPING = 3
    STOPPED = 4
    WARNING = 5

    _VALUES_TO_NAMES = {
        0: "DEAD",
        1: "STARTING",
        2: "ALIVE",
        3: "STOPPING",
        4: "STOPPED",
        5: "WARNING",
    }

    _NAMES_TO_VALUES = {
        "DEAD": 0,
        "STARTING": 1,
        "ALIVE": 2,
        "STOPPING": 3,
        "STOPPED": 4,
        "WARNING": 5,
    }
fix_spec(all_structs)
del all_structs



# Copyright 2013 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import sys
from datetime import datetime, timedelta
from six.moves import http_client

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest

import pytest
from impala.util import (cookie_matches_path, get_cookie_expiry, get_all_cookies,
                         get_all_matching_cookies, get_basic_credentials_for_request_headers)


class ImpalaUtilTests(unittest.TestCase):
    def test_cookie_matches_path(self):
        assert cookie_matches_path({}, '/')
        assert cookie_matches_path({}, '')
        assert cookie_matches_path({}, 'cliservice')
        assert cookie_matches_path({}, '/cliservice')
        assert cookie_matches_path({}, '/cliservice/')

        assert cookie_matches_path({'path': ''}, '/')
        assert cookie_matches_path({'path': '/'}, '')
        assert cookie_matches_path({'path': '/'}, 'clisevice')
        assert cookie_matches_path({'path': '/'}, '/clisevice')

        assert not cookie_matches_path({'path': '/cliservice'}, '')
        assert not cookie_matches_path({'path': '/cliservice'}, '/')
        assert cookie_matches_path({'path': '/cliservice'}, 'cliservice')
        assert cookie_matches_path({'path': 'cliservice'}, '/cliservice')
        assert cookie_matches_path({'path': '/cliservice/'}, '/cliservice/')
        assert cookie_matches_path({'path': '/cliservice'}, 'cliservice/abc/def/ghi')
        assert cookie_matches_path({'path': '/cliservice/'}, 'cliservice/?q=abcd')

        assert not cookie_matches_path({'path': '/a/b/c//'}, '/a')
        assert not cookie_matches_path({'path': '/a/b/c//'}, '/a/b')
        assert cookie_matches_path({'path': '/a/b/c//'}, 'a/b/c')
        assert cookie_matches_path({'path': '/a/b/c//'}, '/a/b/c')
        assert cookie_matches_path({'path': '/a/b/c//'}, 'a/b/c/d')
        assert cookie_matches_path({'path': '/a/b/c//'}, '/a/b/c/d')
        assert cookie_matches_path({'path': '/a/b/c//'}, '/a/b/c/d/e/f/g/h')

    def test_get_cookie_expiry(self):
        assert get_cookie_expiry({}) is None
        assert get_cookie_expiry({'max-age': ''}) is None

        hour = timedelta(hours=1)
        sec = timedelta(seconds=1)
        now = datetime.now()
        assert now <= get_cookie_expiry({'max-age': '0'}) <= now + sec
        now = datetime.now()
        assert now - sec <= get_cookie_expiry({'max-age': '-1'}) <= now

        now = datetime.now()
        assert now - hour <= get_cookie_expiry({'max-age': '-3600'}) <= now - hour + sec
        now = datetime.now()
        assert now + hour <= get_cookie_expiry({'max-age': '3600'}) <= now + hour + sec

        now = datetime.now()
        days2k = timedelta(days=2000)
        assert now - days2k <= get_cookie_expiry({'max-age': '-172800000'}) <= now - days2k + sec
        now = datetime.now()
        assert now + days2k <= get_cookie_expiry({'max-age': '172800000'}) <= now + days2k + sec

    def test_get_matching_cookies(self):
        cookies = get_all_cookies('/path', {})
        assert not cookies

        headers = make_cookie_headers([
            ('c_cookie', 'c_value'),
            ('b_cookie', 'b_value'),
            ('a_cookie', 'a_value')
        ])
        cookies = csort(get_all_cookies('/path', headers))
        assert len(cookies) == 3
        assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
        assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'
        assert cookies[2].key == 'c_cookie' and cookies[2].value == 'c_value'

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path'),
            ('a_cookie', 'a_value;Path=/')
        ])
        cookies = csort(get_all_cookies('/path', headers))
        assert len(cookies) == 3
        assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
        assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'
        assert cookies[2].key == 'c_cookie' and cookies[2].value == 'c_value'

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('B_cookie', 'B_value;Path=/path'),
            ('a_cookie', 'a_value;Path=/path/path2')
        ])
        cookies = csort(get_all_cookies('/path', headers))
        assert len(cookies) == 2
        assert cookies[0].key == 'B_cookie' and cookies[0].value == 'B_value'
        assert cookies[1].key == 'c_cookie' and cookies[1].value == 'c_value'

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path'),
            ('a_cookie', 'a_value;Path=/path')
        ])
        cookies = csort(get_all_cookies('/path/path1', headers))
        assert len(cookies) == 3
        assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
        assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'
        assert cookies[2].key == 'c_cookie' and cookies[2].value == 'c_value'

        headers = make_cookie_headers([
            ('b_cookie', 'b_value;Path=/path1'),
            ('a_cookie', 'a_value;Path=/path2')
        ])
        cookies = get_all_cookies('/path', headers)
        assert not cookies

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path1'),
            ('a_cookie', 'a_value;Path=/path2')
        ])
        cookies = get_all_cookies('/path', headers)
        assert len(cookies) == 1
        assert cookies[0].key == 'c_cookie' and cookies[0].value == 'c_value'

    def test_get_all_matching_cookies(self):
        cookies = get_all_matching_cookies(['a', 'b'], '/path', {})
        assert not cookies

        headers = make_cookie_headers([
            ('c_cookie', 'c_value'),
            ('b_cookie', 'b_value'),
            ('a_cookie', 'a_value')
        ])
        cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
        assert len(cookies) == 2
        assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
        assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'
        cookies = get_all_matching_cookies(['b_cookie', 'a_cookie'], '/path', headers)
        assert len(cookies) == 2
        assert cookies[0].key == 'b_cookie' and cookies[0].value == 'b_value'
        assert cookies[1].key == 'a_cookie' and cookies[1].value == 'a_value'

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path'),
            ('a_cookie', 'a_value;Path=/')
        ])
        cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
        assert len(cookies) == 2
        assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
        assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path'),
            ('a_cookie', 'a_value;Path=/path/path2')
        ])
        cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
        assert len(cookies) == 1
        assert cookies[0].key == 'b_cookie' and cookies[0].value == 'b_value'
        cookies = get_all_matching_cookies(['b_cookie', 'a_cookie'], '/path', headers)
        assert len(cookies) == 1
        assert cookies[0].key == 'b_cookie' and cookies[0].value == 'b_value'

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path'),
            ('a_cookie', 'a_value;Path=/path')
        ])
        cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path/path1',
            headers)
        assert len(cookies) == 2
        assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
        assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path1'),
            ('a_cookie', 'a_value;Path=/path2')
        ])
        cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
        assert not cookies

        headers = make_cookie_headers([
            ('c_cookie', 'c_value;Path=/'),
            ('b_cookie', 'b_value;Path=/path1'),
            ('a_cookie', 'a_value;Path=/path2')
        ])
        cookies = get_all_matching_cookies(['a_cookie', 'b_cookie', 'c_cookie'], '/path',
            headers)
        assert len(cookies) == 1
        assert cookies[0].key == 'c_cookie' and cookies[0].value == 'c_value'

    def test_get_basic_credentials_for_request_headers(self):
        assert get_basic_credentials_for_request_headers(
            user="foo",
            password="bar"
        ) == "Zm9vOmJhcg=="
        assert get_basic_credentials_for_request_headers(
            user="thisisaratherlongusername",
            password="withanotherverylongpasswordresultinginanencodinglongerthan76chars"
        ) == "dGhpc2lzYXJhdGhlcmxvbmd1c2VybmFtZTp3aXRoYW5vdGhlcnZlcnlsb25ncGFzc3dvcmRyZXN1bHRpbmdpbmFuZW5jb2Rpbmdsb25nZXJ0aGFuNzZjaGFycw=="
        assert get_basic_credentials_for_request_headers(
            user="?",
            password="?"
        ) == "Pzo/"


def make_cookie_headers(cookie_vals):
    """Make an HTTPMessage containing Set-Cookie headers for Python 2 or Python 3"""
    if sys.version_info.major == 2:
        # In Python 2 the HTTPMessage is a mimetools.Message object, and the
        # Set-Cookie values all appear in a single header, separated by newlines.
        cookies = ""
        count = 0
        for pair in cookie_vals:
            name = pair[0]
            value = pair[1]
            cookies += name + '=' + value
            if count + 1 < len(cookie_vals):
                # Separate the cookies, unless it is the last cookie.
                cookies += '\n '
            count += 1
        return {'Set-Cookie': cookies}
    else:
        # In Python 3 the HTTPMessage is an email.message.Message, and the
        # Set-Cookie values appear as duplicate headers.
        headers = http_client.HTTPMessage()
        for pair in cookie_vals:
            name = pair[0]
            value = pair[1]
            headers.add_header('Set-Cookie', name + "=" + value)
        return headers

def csort(cookies):
    """Sort list of Morsels as header order is not guaranteed."""
    cookies.sort(key=lambda c: c.key)
    return cookies



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec


from thrift.transport import TTransport
all_structs = []


class TPrimitiveType(object):
    INVALID_TYPE = 0
    NULL_TYPE = 1
    BOOLEAN = 2
    TINYINT = 3
    SMALLINT = 4
    INT = 5
    BIGINT = 6
    FLOAT = 7
    DOUBLE = 8
    DATE = 9
    DATETIME = 10
    TIMESTAMP = 11
    STRING = 12
    BINARY = 13
    DECIMAL = 14
    CHAR = 15
    VARCHAR = 16
    FIXED_UDA_INTERMEDIATE = 17

    _VALUES_TO_NAMES = {
        0: "INVALID_TYPE",
        1: "NULL_TYPE",
        2: "BOOLEAN",
        3: "TINYINT",
        4: "SMALLINT",
        5: "INT",
        6: "BIGINT",
        7: "FLOAT",
        8: "DOUBLE",
        9: "DATE",
        10: "DATETIME",
        11: "TIMESTAMP",
        12: "STRING",
        13: "BINARY",
        14: "DECIMAL",
        15: "CHAR",
        16: "VARCHAR",
        17: "FIXED_UDA_INTERMEDIATE",
    }

    _NAMES_TO_VALUES = {
        "INVALID_TYPE": 0,
        "NULL_TYPE": 1,
        "BOOLEAN": 2,
        "TINYINT": 3,
        "SMALLINT": 4,
        "INT": 5,
        "BIGINT": 6,
        "FLOAT": 7,
        "DOUBLE": 8,
        "DATE": 9,
        "DATETIME": 10,
        "TIMESTAMP": 11,
        "STRING": 12,
        "BINARY": 13,
        "DECIMAL": 14,
        "CHAR": 15,
        "VARCHAR": 16,
        "FIXED_UDA_INTERMEDIATE": 17,
    }


class TTypeNodeType(object):
    SCALAR = 0
    ARRAY = 1
    MAP = 2
    STRUCT = 3

    _VALUES_TO_NAMES = {
        0: "SCALAR",
        1: "ARRAY",
        2: "MAP",
        3: "STRUCT",
    }

    _NAMES_TO_VALUES = {
        "SCALAR": 0,
        "ARRAY": 1,
        "MAP": 2,
        "STRUCT": 3,
    }


class TStmtType(object):
    QUERY = 0
    DDL = 1
    DML = 2
    EXPLAIN = 3
    LOAD = 4
    SET = 5
    ADMIN_FN = 6
    TESTCASE = 7
    CONVERT = 8
    UNKNOWN = 9
    KILL = 10

    _VALUES_TO_NAMES = {
        0: "QUERY",
        1: "DDL",
        2: "DML",
        3: "EXPLAIN",
        4: "LOAD",
        5: "SET",
        6: "ADMIN_FN",
        7: "TESTCASE",
        8: "CONVERT",
        9: "UNKNOWN",
        10: "KILL",
    }

    _NAMES_TO_VALUES = {
        "QUERY": 0,
        "DDL": 1,
        "DML": 2,
        "EXPLAIN": 3,
        "LOAD": 4,
        "SET": 5,
        "ADMIN_FN": 6,
        "TESTCASE": 7,
        "CONVERT": 8,
        "UNKNOWN": 9,
        "KILL": 10,
    }


class TIcebergOperation(object):
    INSERT = 0
    DELETE = 1
    UPDATE = 2
    OPTIMIZE = 3
    MERGE = 4

    _VALUES_TO_NAMES = {
        0: "INSERT",
        1: "DELETE",
        2: "UPDATE",
        3: "OPTIMIZE",
        4: "MERGE",
    }

    _NAMES_TO_VALUES = {
        "INSERT": 0,
        "DELETE": 1,
        "UPDATE": 2,
        "OPTIMIZE": 3,
        "MERGE": 4,
    }


class TExplainLevel(object):
    MINIMAL = 0
    STANDARD = 1
    EXTENDED = 2
    VERBOSE = 3

    _VALUES_TO_NAMES = {
        0: "MINIMAL",
        1: "STANDARD",
        2: "EXTENDED",
        3: "VERBOSE",
    }

    _NAMES_TO_VALUES = {
        "MINIMAL": 0,
        "STANDARD": 1,
        "EXTENDED": 2,
        "VERBOSE": 3,
    }


class TRuntimeFilterMode(object):
    OFF = 0
    LOCAL = 1
    GLOBAL = 2

    _VALUES_TO_NAMES = {
        0: "OFF",
        1: "LOCAL",
        2: "GLOBAL",
    }

    _NAMES_TO_VALUES = {
        "OFF": 0,
        "LOCAL": 1,
        "GLOBAL": 2,
    }


class TPrefetchMode(object):
    NONE = 0
    HT_BUCKET = 1

    _VALUES_TO_NAMES = {
        0: "NONE",
        1: "HT_BUCKET",
    }

    _NAMES_TO_VALUES = {
        "NONE": 0,
        "HT_BUCKET": 1,
    }


class TFunctionCategory(object):
    SCALAR = 0
    AGGREGATE = 1
    ANALYTIC = 2

    _VALUES_TO_NAMES = {
        0: "SCALAR",
        1: "AGGREGATE",
        2: "ANALYTIC",
    }

    _NAMES_TO_VALUES = {
        "SCALAR": 0,
        "AGGREGATE": 1,
        "ANALYTIC": 2,
    }


class TFunctionBinaryType(object):
    BUILTIN = 0
    JAVA = 1
    NATIVE = 2
    IR = 3

    _VALUES_TO_NAMES = {
        0: "BUILTIN",
        1: "JAVA",
        2: "NATIVE",
        3: "IR",
    }

    _NAMES_TO_VALUES = {
        "BUILTIN": 0,
        "JAVA": 1,
        "NATIVE": 2,
        "IR": 3,
    }


class TSortingOrder(object):
    LEXICAL = 0
    ZORDER = 1

    _VALUES_TO_NAMES = {
        0: "LEXICAL",
        1: "ZORDER",
    }

    _NAMES_TO_VALUES = {
        "LEXICAL": 0,
        "ZORDER": 1,
    }


class TScalarType(object):
    """
    Attributes:
     - type
     - len
     - precision
     - scale

    """


    def __init__(self, type=None, len=None, precision=None, scale=None,):
        self.type = type
        self.len = len
        self.precision = precision
        self.scale = scale

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.len = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I32:
                    self.precision = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I32:
                    self.scale = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TScalarType')
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.I32, 1)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        if self.len is not None:
            oprot.writeFieldBegin('len', TType.I32, 2)
            oprot.writeI32(self.len)
            oprot.writeFieldEnd()
        if self.precision is not None:
            oprot.writeFieldBegin('precision', TType.I32, 3)
            oprot.writeI32(self.precision)
            oprot.writeFieldEnd()
        if self.scale is not None:
            oprot.writeFieldBegin('scale', TType.I32, 4)
            oprot.writeI32(self.scale)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.type is None:
            raise TProtocolException(message='Required field type is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TStructField(object):
    """
    Attributes:
     - name
     - comment
     - field_id

    """


    def __init__(self, name=None, comment=None, field_id=None,):
        self.name = name
        self.comment = comment
        self.field_id = field_id

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.name = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.comment = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I32:
                    self.field_id = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TStructField')
        if self.name is not None:
            oprot.writeFieldBegin('name', TType.STRING, 1)
            oprot.writeString(self.name)
            oprot.writeFieldEnd()
        if self.comment is not None:
            oprot.writeFieldBegin('comment', TType.STRING, 2)
            oprot.writeString(self.comment)
            oprot.writeFieldEnd()
        if self.field_id is not None:
            oprot.writeFieldBegin('field_id', TType.I32, 3)
            oprot.writeI32(self.field_id)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.name is None:
            raise TProtocolException(message='Required field name is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TTypeNode(object):
    """
    Attributes:
     - type
     - scalar_type
     - struct_fields

    """


    def __init__(self, type=None, scalar_type=None, struct_fields=None,):
        self.type = type
        self.scalar_type = scalar_type
        self.struct_fields = struct_fields

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.scalar_type = TScalarType()
                    self.scalar_type.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.struct_fields = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = TStructField()
                        _elem5.read(iprot)
                        self.struct_fields.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TTypeNode')
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.I32, 1)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        if self.scalar_type is not None:
            oprot.writeFieldBegin('scalar_type', TType.STRUCT, 2)
            self.scalar_type.write(oprot)
            oprot.writeFieldEnd()
        if self.struct_fields is not None:
            oprot.writeFieldBegin('struct_fields', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.struct_fields))
            for iter6 in self.struct_fields:
                iter6.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.type is None:
            raise TProtocolException(message='Required field type is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TColumnType(object):
    """
    Attributes:
     - types

    """


    def __init__(self, types=None,):
        self.types = types

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.types = []
                    (_etype10, _size7) = iprot.readListBegin()
                    for _i11 in range(_size7):
                        _elem12 = TTypeNode()
                        _elem12.read(iprot)
                        self.types.append(_elem12)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TColumnType')
        if self.types is not None:
            oprot.writeFieldBegin('types', TType.LIST, 1)
            oprot.writeListBegin(TType.STRUCT, len(self.types))
            for iter13 in self.types:
                iter13.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TNetworkAddress(object):
    """
    Attributes:
     - hostname
     - port
     - uds_address

    """


    def __init__(self, hostname=None, port=None, uds_address=None,):
        self.hostname = hostname
        self.port = port
        self.uds_address = uds_address

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.hostname = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.port = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.uds_address = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TNetworkAddress')
        if self.hostname is not None:
            oprot.writeFieldBegin('hostname', TType.STRING, 1)
            oprot.writeString(self.hostname)
            oprot.writeFieldEnd()
        if self.port is not None:
            oprot.writeFieldBegin('port', TType.I32, 2)
            oprot.writeI32(self.port)
            oprot.writeFieldEnd()
        if self.uds_address is not None:
            oprot.writeFieldBegin('uds_address', TType.STRING, 3)
            oprot.writeString(self.uds_address)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.hostname is None:
            raise TProtocolException(message='Required field hostname is unset!')
        if self.port is None:
            raise TProtocolException(message='Required field port is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TAddressesList(object):
    """
    Attributes:
     - addresses

    """


    def __init__(self, addresses=None,):
        self.addresses = addresses

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.addresses = []
                    (_etype17, _size14) = iprot.readListBegin()
                    for _i18 in range(_size14):
                        _elem19 = TNetworkAddress()
                        _elem19.read(iprot)
                        self.addresses.append(_elem19)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TAddressesList')
        if self.addresses is not None:
            oprot.writeFieldBegin('addresses', TType.LIST, 1)
            oprot.writeListBegin(TType.STRUCT, len(self.addresses))
            for iter20 in self.addresses:
                iter20.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.addresses is None:
            raise TProtocolException(message='Required field addresses is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TUniqueId(object):
    """
    Attributes:
     - hi
     - lo

    """


    def __init__(self, hi=None, lo=None,):
        self.hi = hi
        self.lo = lo

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I64:
                    self.hi = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I64:
                    self.lo = iprot.readI64()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TUniqueId')
        if self.hi is not None:
            oprot.writeFieldBegin('hi', TType.I64, 1)
            oprot.writeI64(self.hi)
            oprot.writeFieldEnd()
        if self.lo is not None:
            oprot.writeFieldBegin('lo', TType.I64, 2)
            oprot.writeI64(self.lo)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.hi is None:
            raise TProtocolException(message='Required field hi is unset!')
        if self.lo is None:
            raise TProtocolException(message='Required field lo is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TFunctionName(object):
    """
    Attributes:
     - db_name
     - function_name

    """


    def __init__(self, db_name=None, function_name=None,):
        self.db_name = db_name
        self.function_name = function_name

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.db_name = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.function_name = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TFunctionName')
        if self.db_name is not None:
            oprot.writeFieldBegin('db_name', TType.STRING, 1)
            oprot.writeString(self.db_name)
            oprot.writeFieldEnd()
        if self.function_name is not None:
            oprot.writeFieldBegin('function_name', TType.STRING, 2)
            oprot.writeString(self.function_name)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.function_name is None:
            raise TProtocolException(message='Required field function_name is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TScalarFunction(object):
    """
    Attributes:
     - symbol
     - prepare_fn_symbol
     - close_fn_symbol

    """


    def __init__(self, symbol=None, prepare_fn_symbol=None, close_fn_symbol=None,):
        self.symbol = symbol
        self.prepare_fn_symbol = prepare_fn_symbol
        self.close_fn_symbol = close_fn_symbol

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.prepare_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.close_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TScalarFunction')
        if self.symbol is not None:
            oprot.writeFieldBegin('symbol', TType.STRING, 1)
            oprot.writeString(self.symbol)
            oprot.writeFieldEnd()
        if self.prepare_fn_symbol is not None:
            oprot.writeFieldBegin('prepare_fn_symbol', TType.STRING, 2)
            oprot.writeString(self.prepare_fn_symbol)
            oprot.writeFieldEnd()
        if self.close_fn_symbol is not None:
            oprot.writeFieldBegin('close_fn_symbol', TType.STRING, 3)
            oprot.writeString(self.close_fn_symbol)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.symbol is None:
            raise TProtocolException(message='Required field symbol is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TAggregateFunction(object):
    """
    Attributes:
     - intermediate_type
     - is_analytic_only_fn
     - update_fn_symbol
     - init_fn_symbol
     - serialize_fn_symbol
     - merge_fn_symbol
     - finalize_fn_symbol
     - get_value_fn_symbol
     - remove_fn_symbol
     - ignores_distinct

    """


    def __init__(self, intermediate_type=None, is_analytic_only_fn=None, update_fn_symbol=None, init_fn_symbol=None, serialize_fn_symbol=None, merge_fn_symbol=None, finalize_fn_symbol=None, get_value_fn_symbol=None, remove_fn_symbol=None, ignores_distinct=None,):
        self.intermediate_type = intermediate_type
        self.is_analytic_only_fn = is_analytic_only_fn
        self.update_fn_symbol = update_fn_symbol
        self.init_fn_symbol = init_fn_symbol
        self.serialize_fn_symbol = serialize_fn_symbol
        self.merge_fn_symbol = merge_fn_symbol
        self.finalize_fn_symbol = finalize_fn_symbol
        self.get_value_fn_symbol = get_value_fn_symbol
        self.remove_fn_symbol = remove_fn_symbol
        self.ignores_distinct = ignores_distinct

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.intermediate_type = TColumnType()
                    self.intermediate_type.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.BOOL:
                    self.is_analytic_only_fn = iprot.readBool()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.update_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.init_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRING:
                    self.serialize_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRING:
                    self.merge_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.STRING:
                    self.finalize_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.STRING:
                    self.get_value_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.STRING:
                    self.remove_fn_symbol = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 10:
                if ftype == TType.BOOL:
                    self.ignores_distinct = iprot.readBool()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TAggregateFunction')
        if self.intermediate_type is not None:
            oprot.writeFieldBegin('intermediate_type', TType.STRUCT, 1)
            self.intermediate_type.write(oprot)
            oprot.writeFieldEnd()
        if self.is_analytic_only_fn is not None:
            oprot.writeFieldBegin('is_analytic_only_fn', TType.BOOL, 2)
            oprot.writeBool(self.is_analytic_only_fn)
            oprot.writeFieldEnd()
        if self.update_fn_symbol is not None:
            oprot.writeFieldBegin('update_fn_symbol', TType.STRING, 3)
            oprot.writeString(self.update_fn_symbol)
            oprot.writeFieldEnd()
        if self.init_fn_symbol is not None:
            oprot.writeFieldBegin('init_fn_symbol', TType.STRING, 4)
            oprot.writeString(self.init_fn_symbol)
            oprot.writeFieldEnd()
        if self.serialize_fn_symbol is not None:
            oprot.writeFieldBegin('serialize_fn_symbol', TType.STRING, 5)
            oprot.writeString(self.serialize_fn_symbol)
            oprot.writeFieldEnd()
        if self.merge_fn_symbol is not None:
            oprot.writeFieldBegin('merge_fn_symbol', TType.STRING, 6)
            oprot.writeString(self.merge_fn_symbol)
            oprot.writeFieldEnd()
        if self.finalize_fn_symbol is not None:
            oprot.writeFieldBegin('finalize_fn_symbol', TType.STRING, 7)
            oprot.writeString(self.finalize_fn_symbol)
            oprot.writeFieldEnd()
        if self.get_value_fn_symbol is not None:
            oprot.writeFieldBegin('get_value_fn_symbol', TType.STRING, 8)
            oprot.writeString(self.get_value_fn_symbol)
            oprot.writeFieldEnd()
        if self.remove_fn_symbol is not None:
            oprot.writeFieldBegin('remove_fn_symbol', TType.STRING, 9)
            oprot.writeString(self.remove_fn_symbol)
            oprot.writeFieldEnd()
        if self.ignores_distinct is not None:
            oprot.writeFieldBegin('ignores_distinct', TType.BOOL, 10)
            oprot.writeBool(self.ignores_distinct)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.intermediate_type is None:
            raise TProtocolException(message='Required field intermediate_type is unset!')
        if self.is_analytic_only_fn is None:
            raise TProtocolException(message='Required field is_analytic_only_fn is unset!')
        if self.update_fn_symbol is None:
            raise TProtocolException(message='Required field update_fn_symbol is unset!')
        if self.init_fn_symbol is None:
            raise TProtocolException(message='Required field init_fn_symbol is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TFunction(object):
    """
    Attributes:
     - name
     - binary_type
     - arg_types
     - ret_type
     - has_var_args
     - comment
     - signature
     - hdfs_location
     - scalar_fn
     - aggregate_fn
     - is_persistent
     - last_modified_time

    """


    def __init__(self, name=None, binary_type=None, arg_types=None, ret_type=None, has_var_args=None, comment=None, signature=None, hdfs_location=None, scalar_fn=None, aggregate_fn=None, is_persistent=None, last_modified_time=None,):
        self.name = name
        self.binary_type = binary_type
        self.arg_types = arg_types
        self.ret_type = ret_type
        self.has_var_args = has_var_args
        self.comment = comment
        self.signature = signature
        self.hdfs_location = hdfs_location
        self.scalar_fn = scalar_fn
        self.aggregate_fn = aggregate_fn
        self.is_persistent = is_persistent
        self.last_modified_time = last_modified_time

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.name = TFunctionName()
                    self.name.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.binary_type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.arg_types = []
                    (_etype24, _size21) = iprot.readListBegin()
                    for _i25 in range(_size21):
                        _elem26 = TColumnType()
                        _elem26.read(iprot)
                        self.arg_types.append(_elem26)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRUCT:
                    self.ret_type = TColumnType()
                    self.ret_type.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.BOOL:
                    self.has_var_args = iprot.readBool()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRING:
                    self.comment = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.STRING:
                    self.signature = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.STRING:
                    self.hdfs_location = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.STRUCT:
                    self.scalar_fn = TScalarFunction()
                    self.scalar_fn.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 10:
                if ftype == TType.STRUCT:
                    self.aggregate_fn = TAggregateFunction()
                    self.aggregate_fn.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 11:
                if ftype == TType.BOOL:
                    self.is_persistent = iprot.readBool()
                else:
                    iprot.skip(ftype)
            elif fid == 12:
                if ftype == TType.I64:
                    self.last_modified_time = iprot.readI64()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TFunction')
        if self.name is not None:
            oprot.writeFieldBegin('name', TType.STRUCT, 1)
            self.name.write(oprot)
            oprot.writeFieldEnd()
        if self.binary_type is not None:
            oprot.writeFieldBegin('binary_type', TType.I32, 2)
            oprot.writeI32(self.binary_type)
            oprot.writeFieldEnd()
        if self.arg_types is not None:
            oprot.writeFieldBegin('arg_types', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.arg_types))
            for iter27 in self.arg_types:
                iter27.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.ret_type is not None:
            oprot.writeFieldBegin('ret_type', TType.STRUCT, 4)
            self.ret_type.write(oprot)
            oprot.writeFieldEnd()
        if self.has_var_args is not None:
            oprot.writeFieldBegin('has_var_args', TType.BOOL, 5)
            oprot.writeBool(self.has_var_args)
            oprot.writeFieldEnd()
        if self.comment is not None:
            oprot.writeFieldBegin('comment', TType.STRING, 6)
            oprot.writeString(self.comment)
            oprot.writeFieldEnd()
        if self.signature is not None:
            oprot.writeFieldBegin('signature', TType.STRING, 7)
            oprot.writeString(self.signature)
            oprot.writeFieldEnd()
        if self.hdfs_location is not None:
            oprot.writeFieldBegin('hdfs_location', TType.STRING, 8)
            oprot.writeString(self.hdfs_location)
            oprot.writeFieldEnd()
        if self.scalar_fn is not None:
            oprot.writeFieldBegin('scalar_fn', TType.STRUCT, 9)
            self.scalar_fn.write(oprot)
            oprot.writeFieldEnd()
        if self.aggregate_fn is not None:
            oprot.writeFieldBegin('aggregate_fn', TType.STRUCT, 10)
            self.aggregate_fn.write(oprot)
            oprot.writeFieldEnd()
        if self.is_persistent is not None:
            oprot.writeFieldBegin('is_persistent', TType.BOOL, 11)
            oprot.writeBool(self.is_persistent)
            oprot.writeFieldEnd()
        if self.last_modified_time is not None:
            oprot.writeFieldBegin('last_modified_time', TType.I64, 12)
            oprot.writeI64(self.last_modified_time)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.name is None:
            raise TProtocolException(message='Required field name is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(TScalarType)
TScalarType.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'type', None, None, ),  # 1
    (2, TType.I32, 'len', None, None, ),  # 2
    (3, TType.I32, 'precision', None, None, ),  # 3
    (4, TType.I32, 'scale', None, None, ),  # 4
)
all_structs.append(TStructField)
TStructField.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'name', None, None, ),  # 1
    (2, TType.STRING, 'comment', None, None, ),  # 2
    (3, TType.I32, 'field_id', None, None, ),  # 3
)
all_structs.append(TTypeNode)
TTypeNode.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'type', None, None, ),  # 1
    (2, TType.STRUCT, 'scalar_type', [TScalarType, None], None, ),  # 2
    (3, TType.LIST, 'struct_fields', (TType.STRUCT, [TStructField, None], False), None, ),  # 3
)
all_structs.append(TColumnType)
TColumnType.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'types', (TType.STRUCT, [TTypeNode, None], False), None, ),  # 1
)
all_structs.append(TNetworkAddress)
TNetworkAddress.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'hostname', None, None, ),  # 1
    (2, TType.I32, 'port', None, None, ),  # 2
    (3, TType.STRING, 'uds_address', None, None, ),  # 3
)
all_structs.append(TAddressesList)
TAddressesList.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'addresses', (TType.STRUCT, [TNetworkAddress, None], False), None, ),  # 1
)
all_structs.append(TUniqueId)
TUniqueId.thrift_spec = (
    None,  # 0
    (1, TType.I64, 'hi', None, None, ),  # 1
    (2, TType.I64, 'lo', None, None, ),  # 2
)
all_structs.append(TFunctionName)
TFunctionName.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'db_name', None, None, ),  # 1
    (2, TType.STRING, 'function_name', None, None, ),  # 2
)
all_structs.append(TScalarFunction)
TScalarFunction.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'symbol', None, None, ),  # 1
    (2, TType.STRING, 'prepare_fn_symbol', None, None, ),  # 2
    (3, TType.STRING, 'close_fn_symbol', None, None, ),  # 3
)
all_structs.append(TAggregateFunction)
TAggregateFunction.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'intermediate_type', [TColumnType, None], None, ),  # 1
    (2, TType.BOOL, 'is_analytic_only_fn', None, None, ),  # 2
    (3, TType.STRING, 'update_fn_symbol', None, None, ),  # 3
    (4, TType.STRING, 'init_fn_symbol', None, None, ),  # 4
    (5, TType.STRING, 'serialize_fn_symbol', None, None, ),  # 5
    (6, TType.STRING, 'merge_fn_symbol', None, None, ),  # 6
    (7, TType.STRING, 'finalize_fn_symbol', None, None, ),  # 7
    (8, TType.STRING, 'get_value_fn_symbol', None, None, ),  # 8
    (9, TType.STRING, 'remove_fn_symbol', None, None, ),  # 9
    (10, TType.BOOL, 'ignores_distinct', None, None, ),  # 10
)
all_structs.append(TFunction)
TFunction.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'name', [TFunctionName, None], None, ),  # 1
    (2, TType.I32, 'binary_type', None, None, ),  # 2
    (3, TType.LIST, 'arg_types', (TType.STRUCT, [TColumnType, None], False), None, ),  # 3
    (4, TType.STRUCT, 'ret_type', [TColumnType, None], None, ),  # 4
    (5, TType.BOOL, 'has_var_args', None, None, ),  # 5
    (6, TType.STRING, 'comment', None, None, ),  # 6
    (7, TType.STRING, 'signature', None, None, ),  # 7
    (8, TType.STRING, 'hdfs_location', None, None, ),  # 8
    (9, TType.STRUCT, 'scalar_fn', [TScalarFunction, None], None, ),  # 9
    (10, TType.STRUCT, 'aggregate_fn', [TAggregateFunction, None], None, ),  # 10
    (11, TType.BOOL, 'is_persistent', None, None, ),  # 11
    (12, TType.I64, 'last_modified_time', None, None, ),  # 12
)
fix_spec(all_structs)
del all_structs



# Copyright 2015 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This package's main goal in the past was to clean up references to thrift, because
# we were using thriftpy2 for Py3. This is no longer necessary since upgrading to
# Thrift 0.11.0, as Thrift supports Python 3 since 0.10.0. Now there are only some
# leftover utility classes and functions.

# pylint: disable=wrong-import-position

from __future__ import absolute_import

import base64
import datetime
import getpass
import os
import os.path
from collections import namedtuple
from io import BytesIO

from six.moves import urllib, http_client
import warnings

import six
import ssl
import sys

from impala.error import HttpError
from impala.util import get_basic_credentials_for_request_headers
from impala.util import get_logger_and_init_null
from impala.util import get_all_matching_cookies, get_all_cookies, get_cookie_expiry

# Declare namedtuple for Cookie with named fields - cookie and expiry_time
Cookie = namedtuple('Cookie', ['cookie', 'expiry_time'])

log = get_logger_and_init_null(__name__)


# pylint: disable=import-error,unused-import
# import Apache Thrift code
from thrift.transport.TSocket import TSocket
from thrift.transport.TTransport import (
    TBufferedTransport, TTransportException, TTransportBase)

# import HS2 codegen objects
from impala._thrift_gen.ImpalaService import ImpalaHiveServer2Service
ThriftClient = ImpalaHiveServer2Service.Client


# ImpalaHttpClient is copied from Impala Shell.
# The implementations should be kept in sync as much as possible.
class ImpalaHttpClient(TTransportBase):
  """Http implementation of TTransport base."""

  # When sending requests larger than this size, include the 'Expect: 100-continue' header
  # to indicate to the server to validate the request before reading the contents. This
  # value was chosen to match curl's behavior. See Section 8.2.3 of RFC2616.
  MIN_REQUEST_SIZE_FOR_EXPECT = 1024

  def __init__(self, uri_or_host, port=None, path=None, cafile=None, cert_file=None,
               key_file=None, ssl_context=None, http_cookie_names=None,
               get_user_custom_headers_func=None):
    """ImpalaHttpClient supports two different types of construction:

    ImpalaHttpClient(host, port, path) - deprecated
    ImpalaHttpClient(uri, [port=<n>, path=<s>, cafile=<filename>, cert_file=<filename>,
        key_file=<filename>, ssl_context=<context>, http_cookie_names=<cookienamelist>],
        get_user_custom_headers_func=<function_setting_http_headers>)

    Only the second supports https.  To properly authenticate against the server,
    provide the client's identity by specifying cert_file and key_file.  To properly
    authenticate the server, specify either cafile or ssl_context with a CA defined.
    NOTE: if both cafile and ssl_context are defined, ssl_context will override cafile.
    http_cookie_names is used to specify the list of possible cookie names used for
    cookie-based authentication or session management. If there's only one name in the
    cookie name list, a str value can be specified instead of the list. If a cookie with
    one of these names is returned in an http response by the server or an intermediate
    proxy then it will be included in each subsequent request for the same connection. If
    it is set as wildcards, all cookies in an http response will be preserved.
    The optional get_user_custom_headers_func parameter can be used to add http headers
    to outgoing http messages when using hs2-http protocol. The parameter should be a
    function returning a list of tuples, each tuple containing a key-value pair
    representing the header name and value.
    """
    if port is not None:
      warnings.warn(
        "Please use the ImpalaHttpClient('http{s}://host:port/path') constructor",
        DeprecationWarning,
        stacklevel=2)
      self.host = uri_or_host
      self.port = port
      assert path
      self.path = path
      self.scheme = 'http'
    else:
      parsed = urllib.parse.urlparse(uri_or_host)
      self.scheme = parsed.scheme
      assert self.scheme in ('http', 'https')
      if self.scheme == 'http':
        self.port = parsed.port or http_client.HTTP_PORT
      elif self.scheme == 'https':
        self.port = parsed.port or http_client.HTTPS_PORT
        self.certfile = cert_file
        self.keyfile = key_file
        self.context = ssl.create_default_context(cafile=cafile) \
          if (cafile and not ssl_context) else ssl_context
      self.host = parsed.hostname
      self.path = parsed.path
      if parsed.query:
        self.path += '?%s' % parsed.query
    try:
      proxy = urllib.request.getproxies()[self.scheme]
    except KeyError:
      proxy = None
    else:
      if urllib.request.proxy_bypass(self.host):
        proxy = None
    if proxy:
      parsed = urllib.parse.urlparse(proxy)
      self.realhost = self.host
      self.realport = self.port
      self.host = parsed.hostname
      self.port = parsed.port
      self.proxy_auth = self.basic_proxy_auth_header(parsed)
    else:
      self.realhost = self.realport = self.proxy_auth = None
    if not http_cookie_names:
      # 'http_cookie_names' was explicitly set as an empty value ([], or '') in connect().
      self.__preserve_all_cookies = False
      self.__http_cookie_dict = None
      self.__auth_cookie_names = None
    elif str(http_cookie_names).strip() == "*":
      self.__preserve_all_cookies = True
      self.__http_cookie_dict = dict()
      self.__auth_cookie_names = set()
    else:
      self.__preserve_all_cookies = False
      if isinstance(http_cookie_names, six.string_types):
        http_cookie_names = [http_cookie_names]
      # Build a dictionary that maps cookie name to namedtuple.
      self.__http_cookie_dict = \
          { cn: Cookie(cookie=None, expiry_time=None) for cn in http_cookie_names }
      # Store the auth cookie names in __auth_cookie_names.
      # Assume auth cookie names end with ".auth".
      self.__auth_cookie_names = \
          { cn for cn in http_cookie_names if cn.endswith(".auth") }
    # Set __are_matching_cookies_found as True if matching cookies are found in response.
    self.__are_matching_cookies_found = False
    self.__wbuf = BytesIO()
    self.__http = None
    self.__http_response = None
    self.__timeout = None
    # __custom_headers is used to store HTTP headers which are generated in runtime for
    # new request.
    self.__custom_headers = None
    self.__get_custom_headers_func = None
    # __user_custom_headers is a list of tuples, each tuple contains a key-value pair.
    self.__user_custom_headers = None
    if get_user_custom_headers_func:
        self.__get_user_custom_headers_func = get_user_custom_headers_func
    else:
        self.__get_user_custom_headers_func = None
    # the default user agent if none is provied
    self.__custom_user_agent = 'Python/ImpylaHttpClient'

  @staticmethod
  def basic_proxy_auth_header(proxy):
    if proxy is None or not proxy.username:
      return None
    return "Basic " + get_basic_credentials_for_request_headers(
       user=urllib.parse.unquote(proxy.username),
       password=urllib.parse.unquote(proxy.password),
    )

  def using_proxy(self):
    return self.realhost is not None

  def open(self):
    if self.scheme == 'http':
      self.__http = http_client.HTTPConnection(self.host, self.port,
                                               timeout=self.__timeout)
    elif self.scheme == 'https':
      # (#529) From python 3.12 http_client.HTTPSConnection no longer
      # expects certfile and key_file. Certfile is included in context
      # while key_file can be used in SSLContext.load_verify_locations.
      # As Impyla doesn't support server authentication (#362) both
      # certfile and key_file are expected to be None here.
      if self.certfile or self.keyfile:
        raise NotSupportedError("Server authentication is not supported " +
                                "with HTTP endpoints")
      self.__http = http_client.HTTPSConnection(self.host, self.port,
                                                timeout=self.__timeout,
                                                context=self.context)
    if self.using_proxy():
      self.__http.set_tunnel(self.realhost, self.realport,
                             {"Proxy-Authorization": self.proxy_auth})

  def close(self):
    self.__http.close()
    self.__http = None
    self.__http_response = None

  def isOpen(self):
    return self.__http is not None

  def is_open(self):
    return self.__http is not None

  def setTimeout(self, ms):
    if ms is None:
      self.__timeout = None
    else:
      self.__timeout = ms / 1000.0

  def setCustomHeaders(self, headers):
    self.__custom_headers = headers

  def setCustomUserAgent(self, user_agent):
    self.__custom_user_agent = user_agent

  # Set callback function which generate HTTP headers for a specific auth mechanism.
  def setGetCustomHeadersFunc(self, func):
    self.__get_custom_headers_func = func

  # Update outgoing HTTP headers.
  # This is done by two callback functions, if present
  # __get_custom_headers_func adds headers based on the saved cookies and auth
  # mechanism.
  # __get_user_custom_headers_func adds custom user-supplied http headers.
  def refreshCustomHeaders(self):
    if self.__get_custom_headers_func:
      cookie_header, has_auth_cookie = self.getHttpCookieHeaderForRequest()
      self.__custom_headers = \
          self.__get_custom_headers_func(cookie_header, has_auth_cookie)
    if self.__get_user_custom_headers_func:
       self.__user_custom_headers = \
          self.__get_user_custom_headers_func()

  # Return first value as a cookie list for Cookie header. It's a list of name-value
  # pairs in the form of <cookie-name>=<cookie-value>. Pairs in the list are separated by
  # a semicolon and a space ('; ').
  # Return second value as True if the cookie list contains auth cookie.
  def getHttpCookieHeaderForRequest(self):
    if (self.__http_cookie_dict is None) or not self.__are_matching_cookies_found:
      return None, False
    cookie_headers = []
    has_auth_cookie = False
    for cn, c_tuple in self.__http_cookie_dict.items():
      if c_tuple.cookie:
        if c_tuple.expiry_time and c_tuple.expiry_time <= datetime.datetime.now():
          self.__http_cookie_dict[cn] = Cookie(cookie=None, expiry_time=None)
        else:
          cookie_header = c_tuple.cookie.output(attrs=['value'], header='').strip()
          cookie_headers.append(cookie_header)
          if not has_auth_cookie and self.__auth_cookie_names \
              and cn in self.__auth_cookie_names:
            has_auth_cookie = True
    if not cookie_headers:
      self.__are_matching_cookies_found = False
      return None, False
    else:
      return '; '.join(cookie_headers), has_auth_cookie

  # Extract cookies from response and save those cookies for which the cookie names
  # are in the cookie name list specified in the connect() API.
  def extractHttpCookiesFromResponse(self):
    if self.__preserve_all_cookies:
       matching_cookies = get_all_cookies(self.path, self.headers)
    elif self.__http_cookie_dict is not None:
      matching_cookies = get_all_matching_cookies(
          self.__http_cookie_dict.keys(), self.path, self.headers)
    else:
      matching_cookies = None

    if matching_cookies:
      self.__are_matching_cookies_found = True
      for c in matching_cookies:
        self.__http_cookie_dict[c.key] = Cookie(c, get_cookie_expiry(c))
        if c.key.endswith(".auth"):
          self.__auth_cookie_names.add(c.key)

  # Return True if there are any saved cookies which are sent in previous request.
  def areHttpCookiesSaved(self):
    return self.__are_matching_cookies_found

  # Clean all saved cookies.
  def cleanHttpCookies(self):
    if (self.__http_cookie_dict is not None) and self.__are_matching_cookies_found:
      self.__are_matching_cookies_found = False
      self.__http_cookie_dict = \
          { cn: Cookie(cookie=None, expiry_time=None) for cn in self.__http_cookie_dict }

  def read(self, sz):
    return self.__http_response.read(sz)

  def readBody(self):
    return self.__http_response.read()

  def write(self, buf):
    self.__wbuf.write(buf)

  def flush(self):
    # Send HTTP request and receive response.
    # Return True if the client should retry this method.
    def sendRequestRecvResp(data):
      if self.isOpen():
        self.close()
      self.open()

      # HTTP request
      if self.using_proxy() and self.scheme == "http":
        # need full URL of real host for HTTP proxy here (HTTPS uses CONNECT tunnel)
        self.__http.putrequest('POST', "http://%s:%s%s" %
                               (self.realhost, self.realport, self.path))
      else:
        self.__http.putrequest('POST', self.path)

      # Write headers
      self.__http.putheader('Content-Type', 'application/x-thrift')
      data_len = len(data)
      self.__http.putheader('Content-Length', str(data_len))
      if data_len > ImpalaHttpClient.MIN_REQUEST_SIZE_FOR_EXPECT:
        # Add the 'Expect' header to large requests. Note that we do not explicitly wait
        # for the '100 continue' response before sending the data - HTTPConnection simply
        # ignores these types of responses, but we'll get the right behavior anyways.
        self.__http.putheader("Expect", "100-continue")
      if self.using_proxy() and self.scheme == "http" and self.proxy_auth is not None:
        self.__http.putheader("Proxy-Authorization", self.proxy_auth)

      self.refreshCustomHeaders()
      if not self.__custom_headers or 'User-Agent' not in self.__custom_headers:
        user_agent = self.__custom_user_agent
        script = os.path.basename(sys.argv[0])
        if script:
          user_agent = '%s (%s)' % (user_agent, urllib.parse.quote(script))
        self.__http.putheader('User-Agent', user_agent)

      if self.__custom_headers:
        for key, val in six.iteritems(self.__custom_headers):
          self.__http.putheader(key, val)
      if self.__user_custom_headers:
        for key, val in self.__user_custom_headers:
          self.__http.putheader(key, val)

      self.__http.endheaders()

      # Write payload
      self.__http.send(data)

      # Get reply to flush the request
      self.__http_response = self.__http.getresponse()
      self.code = self.__http_response.status
      self.message = self.__http_response.reason
      self.headers = self.__http_response.msg
      # A '401 Unauthorized' response might mean that we tried cookie-based
      # authentication with one or more expired cookies.
      # Delete the cookies and try again.
      if self.code == 401 and self.areHttpCookiesSaved():
        self.cleanHttpCookies()
        return True
      else:
        self.extractHttpCookiesFromResponse()
        return False

    # Pull data out of buffer
    data = self.__wbuf.getvalue()
    self.__wbuf = BytesIO()

    retry = sendRequestRecvResp(data)
    if retry:
      log.debug('Received "401 Unauthorized" response. '
                'Delete HTTP cookies and then retry.')
      sendRequestRecvResp(data)

    if self.code >= 300:
      # Report any http response code that is not 1XX (informational response) or
      # 2XX (successful).
      body = self.readBody()
      raise HttpError(self.code, self.message, body, self.headers)


def get_socket(host, port, use_ssl, ca_cert):
    # based on the Impala shell impl
    log.debug('get_socket: host=%s port=%s use_ssl=%s ca_cert=%s',
              host, port, use_ssl, ca_cert)

    if use_ssl:
        from thrift.transport.TSSLSocket import TSSLSocket

        # This copies the solution in IMPALA-11343.
        # TODO: remove once Thrit 0.17.0 is released
        class ImpalaTSSLSocket(TSSLSocket):
          # THRIFT-5595: override TSocket.isOpen because it's broken for TSSLSocket
          def isOpen(self):
            return self.handle is not None

        if ca_cert is None:
            return ImpalaTSSLSocket(host, port, validate=False)
        else:
            return ImpalaTSSLSocket(host, port, validate=True, ca_certs=ca_cert)
    else:
        return TSocket(host, port)


def get_http_transport(host, port, http_path, timeout=None, use_ssl=False,
                       ca_cert=None, auth_mechanism='NOSASL', user=None,
                       password=None, kerberos_host=None, kerberos_service_name=None,
                       http_cookie_names=None, jwt=None, user_agent=None,
                       get_user_custom_headers_func=None):
    host_url = "[%s]" % host if ":" in host else host # add brackets for ipv6 address
    # TODO: support timeout
    if timeout is not None:
        log.error('get_http_transport does not support a timeout')
    if use_ssl:
        ssl_ctx = ssl.create_default_context(cafile=ca_cert)
        if ca_cert:
          ssl_ctx.verify_mode = ssl.CERT_REQUIRED
        else:
          ssl_ctx.check_hostname = False  # Mandated by the SSL lib for CERT_NONE mode.
          ssl_ctx.verify_mode = ssl.CERT_NONE

        url = 'https://%s:%s/%s' % (host_url, port, http_path)
        log.debug('get_http_transport url=%s', url)
        # TODO(#362): Add server authentication with thrift 0.12.
        transport = ImpalaHttpClient(
            url, ssl_context=ssl_ctx,
            http_cookie_names=http_cookie_names,
            get_user_custom_headers_func=get_user_custom_headers_func)
    else:
        url = 'http://%s:%s/%s' % (host_url, port, http_path)
        log.debug('get_http_transport url=%s', url)
        transport = ImpalaHttpClient(
            url, http_cookie_names=http_cookie_names,
            get_user_custom_headers_func=get_user_custom_headers_func)

    # set custom user agent if provided by user
    if user_agent:
        transport.setCustomUserAgent(user_agent)

    if auth_mechanism in ['PLAIN', 'LDAP']:
        # Set defaults for PLAIN SASL / LDAP connections.
        if user is None:
            user = getpass.getuser()
            log.debug('get_http_transport: user=%s', user)
        if password is None:
            if auth_mechanism == 'LDAP':
                password = ''
            else:
                # PLAIN always requires a password for HS2.
                password = 'password'
            log.debug('get_http_transport: password=%s', password)
        else:
            log.debug('get_http_transport: password=fuggetaboutit')
        auth_mechanism = 'PLAIN'  # sasl doesn't know mechanism LDAP
        # Set the BASIC auth header
        auth = get_basic_credentials_for_request_headers(user, password)

        def get_custom_headers(cookie_header, has_auth_cookie):
            custom_headers = {}
            if cookie_header:
                log.debug('add cookies to HTTP header')
                custom_headers['Cookie'] = cookie_header
            # Add the 'Authorization' header to request even if the auth cookie is
            # present to avoid a round trip in case the cookie is expired when server
            # receive the request. Since the 'auth' value is calculated once, so it
            # won't cause a performance issue.
            custom_headers['Authorization'] = "Basic " + auth
            return custom_headers

        transport.setGetCustomHeadersFunc(get_custom_headers)

    elif auth_mechanism == 'GSSAPI':
        # For GSSAPI over http we need to dynamically generate custom request headers.
        def get_custom_headers(cookie_header, has_auth_cookie):
            import kerberos
            custom_headers = {}
            if cookie_header:
                log.debug('add cookies to HTTP header')
                custom_headers['Cookie'] = cookie_header
            if not has_auth_cookie:
                _, krb_context = kerberos.authGSSClientInit("%s@%s" %
                                    (kerberos_service_name, kerberos_host))
                kerberos.authGSSClientStep(krb_context, "")
                negotiate_details = kerberos.authGSSClientResponse(krb_context)
                custom_headers['Authorization'] = "Negotiate " + negotiate_details
            return custom_headers

        transport.setGetCustomHeadersFunc(get_custom_headers)

    elif auth_mechanism == 'JWT':
        # For JWT authentication, the JWT is sent on the Authorization Bearer
        # HTTP header.
        def get_custom_headers(cookie_header, has_auth_cookie):
            custom_headers = {}
            if cookie_header:
                log.debug('add cookies to HTTP header')
                custom_headers['Cookie'] = cookie_header

            custom_headers['Authorization'] = "Bearer " + jwt
            return custom_headers

        transport.setGetCustomHeadersFunc(get_custom_headers)

    elif auth_mechanism == 'NOSASL':
        def get_custom_headers(cookie_header, has_auth_cookie):
            custom_headers = {}
            if cookie_header:
                log.debug('add cookies to HTTP header')
                custom_headers['Cookie'] = cookie_header
            return custom_headers

        transport.setGetCustomHeadersFunc(get_custom_headers)

    # Without buffering Thrift would call socket.recv() each time it deserializes
    # something (e.g. a member in a struct).
    transport = TBufferedTransport(transport)
    return transport


def get_transport(socket, host, kerberos_service_name, auth_mechanism='NOSASL',
                  user=None, password=None):
    """
    Creates a new Thrift Transport using the specified auth_mechanism.
    Supported auth_mechanisms are:
    - None or 'NOSASL' - returns simple buffered transport (default)
    - 'PLAIN'  - returns a SASL transport with the PLAIN mechanism
    - 'GSSAPI' - returns a SASL transport with the GSSAPI mechanism
    """
    log.debug('get_transport: socket=%s host=%s kerberos_service_name=%s '
              'auth_mechanism=%s user=%s password=fuggetaboutit', socket, host,
              kerberos_service_name, auth_mechanism, user)

    if auth_mechanism == 'NOSASL':
        return TBufferedTransport(socket)

    # Set defaults for PLAIN SASL / LDAP connections.
    if auth_mechanism in ['LDAP', 'PLAIN']:
        if user is None:
            user = getpass.getuser()
            log.debug('get_transport: user=%s', user)
        if password is None:
            if auth_mechanism == 'LDAP':
                password = ''
            else:
                # PLAIN always requires a password for HS2.
                password = 'password'
            log.debug('get_transport: password=%s', password)
        auth_mechanism = 'PLAIN'  # sasl doesn't know mechanism LDAP
    # Initializes a sasl client
    from thrift_sasl import TSaslClientTransport
    from impala.sasl_compat import PureSASLClient

    def sasl_factory():
        return PureSASLClient(host, username=user, password=password,
                              service=kerberos_service_name)

    return TSaslClientTransport(sasl_factory, auth_mechanism, socket)



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import impala._thrift_gen.beeswax.BeeswaxService
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(impala._thrift_gen.beeswax.BeeswaxService.Iface):
    def Cancel(self, query_id):
        """
        Parameters:
         - query_id

        """
        pass

    def ResetCatalog(self):
        pass

    def ResetTable(self, request):
        """
        Parameters:
         - request

        """
        pass

    def GetRuntimeProfile(self, query_id):
        """
        Parameters:
         - query_id

        """
        pass

    def CloseInsert(self, handle):
        """
        Parameters:
         - handle

        """
        pass

    def PingImpalaService(self):
        pass

    def GetExecSummary(self, handle):
        """
        Parameters:
         - handle

        """
        pass


class Client(impala._thrift_gen.beeswax.BeeswaxService.Client, Iface):
    def __init__(self, iprot, oprot=None):
        impala._thrift_gen.beeswax.BeeswaxService.Client.__init__(self, iprot, oprot)

    def Cancel(self, query_id):
        """
        Parameters:
         - query_id

        """
        self.send_Cancel(query_id)
        return self.recv_Cancel()

    def send_Cancel(self, query_id):
        self._oprot.writeMessageBegin('Cancel', TMessageType.CALL, self._seqid)
        args = Cancel_args()
        args.query_id = query_id
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_Cancel(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = Cancel_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.error is not None:
            raise result.error
        raise TApplicationException(TApplicationException.MISSING_RESULT, "Cancel failed: unknown result")

    def ResetCatalog(self):
        self.send_ResetCatalog()
        return self.recv_ResetCatalog()

    def send_ResetCatalog(self):
        self._oprot.writeMessageBegin('ResetCatalog', TMessageType.CALL, self._seqid)
        args = ResetCatalog_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_ResetCatalog(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = ResetCatalog_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "ResetCatalog failed: unknown result")

    def ResetTable(self, request):
        """
        Parameters:
         - request

        """
        self.send_ResetTable(request)
        return self.recv_ResetTable()

    def send_ResetTable(self, request):
        self._oprot.writeMessageBegin('ResetTable', TMessageType.CALL, self._seqid)
        args = ResetTable_args()
        args.request = request
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_ResetTable(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = ResetTable_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "ResetTable failed: unknown result")

    def GetRuntimeProfile(self, query_id):
        """
        Parameters:
         - query_id

        """
        self.send_GetRuntimeProfile(query_id)
        return self.recv_GetRuntimeProfile()

    def send_GetRuntimeProfile(self, query_id):
        self._oprot.writeMessageBegin('GetRuntimeProfile', TMessageType.CALL, self._seqid)
        args = GetRuntimeProfile_args()
        args.query_id = query_id
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_GetRuntimeProfile(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = GetRuntimeProfile_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.error is not None:
            raise result.error
        raise TApplicationException(TApplicationException.MISSING_RESULT, "GetRuntimeProfile failed: unknown result")

    def CloseInsert(self, handle):
        """
        Parameters:
         - handle

        """
        self.send_CloseInsert(handle)
        return self.recv_CloseInsert()

    def send_CloseInsert(self, handle):
        self._oprot.writeMessageBegin('CloseInsert', TMessageType.CALL, self._seqid)
        args = CloseInsert_args()
        args.handle = handle
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_CloseInsert(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = CloseInsert_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.error is not None:
            raise result.error
        if result.error2 is not None:
            raise result.error2
        raise TApplicationException(TApplicationException.MISSING_RESULT, "CloseInsert failed: unknown result")

    def PingImpalaService(self):
        self.send_PingImpalaService()
        return self.recv_PingImpalaService()

    def send_PingImpalaService(self):
        self._oprot.writeMessageBegin('PingImpalaService', TMessageType.CALL, self._seqid)
        args = PingImpalaService_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_PingImpalaService(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = PingImpalaService_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "PingImpalaService failed: unknown result")

    def GetExecSummary(self, handle):
        """
        Parameters:
         - handle

        """
        self.send_GetExecSummary(handle)
        return self.recv_GetExecSummary()

    def send_GetExecSummary(self, handle):
        self._oprot.writeMessageBegin('GetExecSummary', TMessageType.CALL, self._seqid)
        args = GetExecSummary_args()
        args.handle = handle
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_GetExecSummary(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = GetExecSummary_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.error is not None:
            raise result.error
        if result.error2 is not None:
            raise result.error2
        raise TApplicationException(TApplicationException.MISSING_RESULT, "GetExecSummary failed: unknown result")


class Processor(impala._thrift_gen.beeswax.BeeswaxService.Processor, Iface, TProcessor):
    def __init__(self, handler):
        impala._thrift_gen.beeswax.BeeswaxService.Processor.__init__(self, handler)
        self._processMap["Cancel"] = Processor.process_Cancel
        self._processMap["ResetCatalog"] = Processor.process_ResetCatalog
        self._processMap["ResetTable"] = Processor.process_ResetTable
        self._processMap["GetRuntimeProfile"] = Processor.process_GetRuntimeProfile
        self._processMap["CloseInsert"] = Processor.process_CloseInsert
        self._processMap["PingImpalaService"] = Processor.process_PingImpalaService
        self._processMap["GetExecSummary"] = Processor.process_GetExecSummary
        self._on_message_begin = None

    def on_message_begin(self, func):
        self._on_message_begin = func

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if self._on_message_begin:
            self._on_message_begin(name, type, seqid)
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_Cancel(self, seqid, iprot, oprot):
        args = Cancel_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = Cancel_result()
        try:
            result.success = self._handler.Cancel(args.query_id)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except impala._thrift_gen.beeswax.ttypes.BeeswaxException as error:
            msg_type = TMessageType.REPLY
            result.error = error
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("Cancel", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_ResetCatalog(self, seqid, iprot, oprot):
        args = ResetCatalog_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = ResetCatalog_result()
        try:
            result.success = self._handler.ResetCatalog()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("ResetCatalog", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_ResetTable(self, seqid, iprot, oprot):
        args = ResetTable_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = ResetTable_result()
        try:
            result.success = self._handler.ResetTable(args.request)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("ResetTable", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_GetRuntimeProfile(self, seqid, iprot, oprot):
        args = GetRuntimeProfile_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = GetRuntimeProfile_result()
        try:
            result.success = self._handler.GetRuntimeProfile(args.query_id)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except impala._thrift_gen.beeswax.ttypes.BeeswaxException as error:
            msg_type = TMessageType.REPLY
            result.error = error
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("GetRuntimeProfile", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_CloseInsert(self, seqid, iprot, oprot):
        args = CloseInsert_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = CloseInsert_result()
        try:
            result.success = self._handler.CloseInsert(args.handle)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except impala._thrift_gen.beeswax.ttypes.QueryNotFoundException as error:
            msg_type = TMessageType.REPLY
            result.error = error
        except impala._thrift_gen.beeswax.ttypes.BeeswaxException as error2:
            msg_type = TMessageType.REPLY
            result.error2 = error2
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("CloseInsert", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_PingImpalaService(self, seqid, iprot, oprot):
        args = PingImpalaService_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = PingImpalaService_result()
        try:
            result.success = self._handler.PingImpalaService()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("PingImpalaService", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_GetExecSummary(self, seqid, iprot, oprot):
        args = GetExecSummary_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = GetExecSummary_result()
        try:
            result.success = self._handler.GetExecSummary(args.handle)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except impala._thrift_gen.beeswax.ttypes.QueryNotFoundException as error:
            msg_type = TMessageType.REPLY
            result.error = error
        except impala._thrift_gen.beeswax.ttypes.BeeswaxException as error2:
            msg_type = TMessageType.REPLY
            result.error2 = error2
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("GetExecSummary", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class Cancel_args(object):
    """
    Attributes:
     - query_id

    """


    def __init__(self, query_id=None,):
        self.query_id = query_id

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.query_id = impala._thrift_gen.beeswax.ttypes.QueryHandle()
                    self.query_id.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Cancel_args')
        if self.query_id is not None:
            oprot.writeFieldBegin('query_id', TType.STRUCT, 1)
            self.query_id.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(Cancel_args)
Cancel_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'query_id', [impala._thrift_gen.beeswax.ttypes.QueryHandle, None], None, ),  # 1
)


class Cancel_result(object):
    """
    Attributes:
     - success
     - error

    """


    def __init__(self, success=None, error=None,):
        self.success = success
        self.error = error

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = impala._thrift_gen.Status.ttypes.TStatus()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.error = impala._thrift_gen.beeswax.ttypes.BeeswaxException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Cancel_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.error is not None:
            oprot.writeFieldBegin('error', TType.STRUCT, 1)
            self.error.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(Cancel_result)
Cancel_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [impala._thrift_gen.Status.ttypes.TStatus, None], None, ),  # 0
    (1, TType.STRUCT, 'error', [impala._thrift_gen.beeswax.ttypes.BeeswaxException, None], None, ),  # 1
)


class ResetCatalog_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('ResetCatalog_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(ResetCatalog_args)
ResetCatalog_args.thrift_spec = (
)


class ResetCatalog_result(object):
    """
    Attributes:
     - success

    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = impala._thrift_gen.Status.ttypes.TStatus()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('ResetCatalog_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(ResetCatalog_result)
ResetCatalog_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [impala._thrift_gen.Status.ttypes.TStatus, None], None, ),  # 0
)


class ResetTable_args(object):
    """
    Attributes:
     - request

    """


    def __init__(self, request=None,):
        self.request = request

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.request = TResetTableReq()
                    self.request.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('ResetTable_args')
        if self.request is not None:
            oprot.writeFieldBegin('request', TType.STRUCT, 1)
            self.request.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(ResetTable_args)
ResetTable_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'request', [TResetTableReq, None], None, ),  # 1
)


class ResetTable_result(object):
    """
    Attributes:
     - success

    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = impala._thrift_gen.Status.ttypes.TStatus()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('ResetTable_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(ResetTable_result)
ResetTable_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [impala._thrift_gen.Status.ttypes.TStatus, None], None, ),  # 0
)


class GetRuntimeProfile_args(object):
    """
    Attributes:
     - query_id

    """


    def __init__(self, query_id=None,):
        self.query_id = query_id

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.query_id = impala._thrift_gen.beeswax.ttypes.QueryHandle()
                    self.query_id.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetRuntimeProfile_args')
        if self.query_id is not None:
            oprot.writeFieldBegin('query_id', TType.STRUCT, 1)
            self.query_id.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetRuntimeProfile_args)
GetRuntimeProfile_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'query_id', [impala._thrift_gen.beeswax.ttypes.QueryHandle, None], None, ),  # 1
)


class GetRuntimeProfile_result(object):
    """
    Attributes:
     - success
     - error

    """


    def __init__(self, success=None, error=None,):
        self.success = success
        self.error = error

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRING:
                    self.success = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.error = impala._thrift_gen.beeswax.ttypes.BeeswaxException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetRuntimeProfile_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRING, 0)
            oprot.writeString(self.success)
            oprot.writeFieldEnd()
        if self.error is not None:
            oprot.writeFieldBegin('error', TType.STRUCT, 1)
            self.error.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetRuntimeProfile_result)
GetRuntimeProfile_result.thrift_spec = (
    (0, TType.STRING, 'success', None, None, ),  # 0
    (1, TType.STRUCT, 'error', [impala._thrift_gen.beeswax.ttypes.BeeswaxException, None], None, ),  # 1
)


class CloseInsert_args(object):
    """
    Attributes:
     - handle

    """


    def __init__(self, handle=None,):
        self.handle = handle

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.handle = impala._thrift_gen.beeswax.ttypes.QueryHandle()
                    self.handle.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('CloseInsert_args')
        if self.handle is not None:
            oprot.writeFieldBegin('handle', TType.STRUCT, 1)
            self.handle.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(CloseInsert_args)
CloseInsert_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'handle', [impala._thrift_gen.beeswax.ttypes.QueryHandle, None], None, ),  # 1
)


class CloseInsert_result(object):
    """
    Attributes:
     - success
     - error
     - error2

    """


    def __init__(self, success=None, error=None, error2=None,):
        self.success = success
        self.error = error
        self.error2 = error2

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = TDmlResult()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.error = impala._thrift_gen.beeswax.ttypes.QueryNotFoundException.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.error2 = impala._thrift_gen.beeswax.ttypes.BeeswaxException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('CloseInsert_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.error is not None:
            oprot.writeFieldBegin('error', TType.STRUCT, 1)
            self.error.write(oprot)
            oprot.writeFieldEnd()
        if self.error2 is not None:
            oprot.writeFieldBegin('error2', TType.STRUCT, 2)
            self.error2.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(CloseInsert_result)
CloseInsert_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [TDmlResult, None], None, ),  # 0
    (1, TType.STRUCT, 'error', [impala._thrift_gen.beeswax.ttypes.QueryNotFoundException, None], None, ),  # 1
    (2, TType.STRUCT, 'error2', [impala._thrift_gen.beeswax.ttypes.BeeswaxException, None], None, ),  # 2
)


class PingImpalaService_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('PingImpalaService_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(PingImpalaService_args)
PingImpalaService_args.thrift_spec = (
)


class PingImpalaService_result(object):
    """
    Attributes:
     - success

    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = TPingImpalaServiceResp()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('PingImpalaService_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(PingImpalaService_result)
PingImpalaService_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [TPingImpalaServiceResp, None], None, ),  # 0
)


class GetExecSummary_args(object):
    """
    Attributes:
     - handle

    """


    def __init__(self, handle=None,):
        self.handle = handle

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.handle = impala._thrift_gen.beeswax.ttypes.QueryHandle()
                    self.handle.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetExecSummary_args')
        if self.handle is not None:
            oprot.writeFieldBegin('handle', TType.STRUCT, 1)
            self.handle.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetExecSummary_args)
GetExecSummary_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'handle', [impala._thrift_gen.beeswax.ttypes.QueryHandle, None], None, ),  # 1
)


class GetExecSummary_result(object):
    """
    Attributes:
     - success
     - error
     - error2

    """


    def __init__(self, success=None, error=None, error2=None,):
        self.success = success
        self.error = error
        self.error2 = error2

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = impala._thrift_gen.ExecStats.ttypes.TExecSummary()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.error = impala._thrift_gen.beeswax.ttypes.QueryNotFoundException.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.error2 = impala._thrift_gen.beeswax.ttypes.BeeswaxException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetExecSummary_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.error is not None:
            oprot.writeFieldBegin('error', TType.STRUCT, 1)
            self.error.write(oprot)
            oprot.writeFieldEnd()
        if self.error2 is not None:
            oprot.writeFieldBegin('error2', TType.STRUCT, 2)
            self.error2.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetExecSummary_result)
GetExecSummary_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [impala._thrift_gen.ExecStats.ttypes.TExecSummary, None], None, ),  # 0
    (1, TType.STRUCT, 'error', [impala._thrift_gen.beeswax.ttypes.QueryNotFoundException, None], None, ),  # 1
    (2, TType.STRUCT, 'error2', [impala._thrift_gen.beeswax.ttypes.BeeswaxException, None], None, ),  # 2
)
fix_spec(all_structs)
del all_structs



# Copyright 2014 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from sqlalchemy.engine import create_engine
from sqlalchemy import Table, Column, select, insert, text
from sqlalchemy.schema import MetaData, CreateTable

from impala.sqlalchemy import STRING, INT, DOUBLE, TINYINT, DATE, VARCHAR
from impala.tests.util import ImpylaTestEnv
import pandas as pd

TEST_ENV = ImpylaTestEnv()


def create_partitioned_test_table():
    metadata = MetaData()
    # TODO: add other types to this table (e.g., functional.all_types)
    return Table("mytable",
                    metadata,
                    Column('col1', STRING),
                    Column('col2', TINYINT),
                    Column('col3', INT),
                    Column('col4', DOUBLE),
                    Column('col5', DATE),
                    Column('col6', VARCHAR(10)),
                    impala_partitioned_by='(part_col STRING)',
                    impala_stored_as='PARQUET',
                    impala_table_properties={
                      'transactional': 'true',
                      'transactional_properties': 'insert_only'
                    })

def create_simple_test_table():
  metadata = MetaData()
  return Table("mytable",
               metadata,
               Column('col1', STRING),
               Column('col2', TINYINT),
               Column('col3', INT),
               Column('col4', DOUBLE)
               )

def create_test_engine(diealect):
    host = "[%s]" % TEST_ENV.host if ":" in TEST_ENV.host else TEST_ENV.host
    return create_engine('{0}://{1}:{2}'.format(diealect, host, TEST_ENV.port))

def test_sqlalchemy_impala_compilation():
    engine = create_test_engine("impala")
    observed = CreateTable(create_partitioned_test_table()).compile(engine)
    # The DATE column type of 'col5' will be replaced with TIMESTAMP.
    expected = ('\nCREATE TABLE mytable (\n\tcol1 STRING, \n\tcol2 TINYINT, '
                '\n\tcol3 INT, \n\tcol4 DOUBLE, \n\tcol5 TIMESTAMP, \n\tcol6 VARCHAR(10)\n)'
                '\nPARTITIONED BY (part_col STRING)\nSTORED AS PARQUET\n'
                "TBLPROPERTIES ('transactional' = 'true', "
                "'transactional_properties' = 'insert_only')\n\n")
    assert expected == str(observed)


def test_sqlalchemy_impala4_compilation():
    engine = create_test_engine("impala4")
    observed = CreateTable(create_partitioned_test_table()).compile(engine)
    # The DATE column type of 'col5' will be left as is.
    expected = ('\nCREATE TABLE mytable (\n\tcol1 STRING, \n\tcol2 TINYINT, '
                '\n\tcol3 INT, \n\tcol4 DOUBLE, \n\tcol5 DATE, \n\tcol6 VARCHAR(10)\n)'
                '\nPARTITIONED BY (part_col STRING)\nSTORED AS PARQUET\n'
                "TBLPROPERTIES ('transactional' = 'true', "
                "'transactional_properties' = 'insert_only')\n\n")
    assert expected == str(observed)

def test_sqlalchemy_multiinsert():
    engine = create_test_engine("impala4")
    table = create_simple_test_table()
    # TODO: Creating a non partitioned table as I am not sure about how to insert to
    #       a partitioned table in SQL alchemy
    create_table_stmt = CreateTable(table)

    data = [
        {"col1": "a", "col2": 1, "col3": 1, "col4": 1.0},
        {"col1": "b", "col2": 2, "col3": 3, "col4": 2.0}
    ]
    insert_stmt = insert(table).values(data).compile(engine)
    expected_insert = 'INSERT INTO mytable (col1, col2, col3, col4) VALUES '\
        '(%(col1_m0)s, %(col2_m0)s, %(col3_m0)s, %(col4_m0)s), '\
        '(%(col1_m1)s, %(col2_m1)s, %(col3_m1)s, %(col4_m1)s)'
    assert expected_insert == str(insert_stmt)

    with engine.connect() as conn:
        conn.execute(create_table_stmt)
        try:
            conn.execute(insert_stmt)
            result = conn.execute(select(table.c).order_by(table.c.col1)).fetchall()
            expected_result = [('a', 1, 1, 1.0), ('b', 2, 3, 2.0)]
            assert expected_result == result
        finally:
            table.drop(conn)

def test_pandas_dataframe_to_sql():
    engine = create_test_engine("impala")
    # Creating a sample dataframe to push to the DB.
    df = pd.DataFrame([[1, 2, 3], [4, 5, 6], [7, 8, 9]], columns=['a', 'b', 'c'])

    with engine.connect() as conn:
        try:
            df.to_sql('test_table', conn, if_exists='replace', index=False)
            table = pd.read_sql('DESCRIBE test_table', conn)
            columns = table['name'].tolist()
            assert ['a', 'b', 'c'] == columns

        finally:
            conn.execute(text('DROP TABLE test_table'))


#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

from .ttypes import *
TErrorMessage = [
    "",
    "<UNUSED>",
    "$0",
    "Cancelled",
    "$0",
    "$0",
    "$0",
    "$0",
    "$0",
    "$0",
    "Parquet files should not be split into multiple hdfs-blocks. file=$0",
    "Column metadata states there are $0 values, but read $1 values from column $2. file=$3",
    "(unused)",
    "ParquetScanner: reached EOF while deserializing data page header. file=$0",
    "Metadata states that in group $0($1) there are $2 rows, but $3 rows were read.",
    "(unused)",
    "File '$0' column '$1' does not have the decimal precision set.",
    "File '$0' column '$1' has a precision that does not match the table metadata precision. File metadata precision: $2, table metadata precision: $3.",
    "File '$0' column '$1' does not have converted type set to DECIMAL",
    "File '$0' column '$1' contains decimal data but the table metadata has type $2",
    "Problem parsing file $0 at $1$2",
    "Decompressor: block size is too big.  Data is likely corrupt. Size: $0",
    "Decompressor: invalid compressed length.  Data is likely corrupt.",
    "Snappy: GetUncompressedLength failed",
    "SnappyBlock: RawUncompress failed",
    "Snappy: Decompressed size is not correct.",
    "Reserved resource size ($0) is larger than query mem limit ($1), and will be restricted to $1. Configure the reservation size by setting RM_INITIAL_MEM.",
    "Cannot perform join at hash join node with id $0. The input data was partitioned the maximum number of $1 times. This could mean there is significant skew in the data or the memory limit is set too low.",
    "Cannot perform aggregation at hash aggregation node with id $0. The input data was partitioned the maximum number of $1 times. This could mean there is significant skew in the data or the memory limit is set too low.",
    "Builtin '$0' with symbol '$1' does not exist. Verify that all your impalads are the same version.",
    "RPC Error: $0",
    "RPC recv timed out: dest address: $0, rpc: $1",
    "Failed to verify function $0 from LLVM module $1, see log for more details.",
    "File $0 corrupt. RLE level data bytes = $1",
    "Column '$0' has conflicting Avro decimal types. Table schema $1: $2, file schema $1: $3",
    "Column '$0' has conflicting Avro decimal types. Declared $1: $2, $1 in table's Avro schema: $3",
    "Unresolvable types for column '$0': table type: $1, file type: $2",
    "Unresolvable types for column '$0': declared column type: $1, table's Avro schema type: $2",
    "Field $0 is missing from file and default values of type $1 are not yet supported.",
    "Inconsistent table metadata. Mismatch between column definition and Avro schema: cannot read field $0 because there are only $1 fields.",
    "Field $0 is missing from file and does not have a default value.",
    "Field $0 is nullable in the file schema but not the table schema.",
    "Inconsistent table metadata. Field $0 is not a record in the Avro schema.",
    "Could not read definition level, even though metadata states there are $0 values remaining in data page. file=$1",
    "Mismatched number of values in column index $0 ($1 vs. $2). file=$3",
    "File '$0' is corrupt: error decoding dictionary-encoded value of type $1 at offset $2",
    "SSL private-key password command ('$0') failed with error: $1",
    "The SSL certificate path is blank",
    "The SSL private key path is blank",
    "The SSL certificate file does not exist at path $0",
    "The SSL private key file does not exist at path $0",
    "SSL socket creation failed: $0",
    "Memory allocation of $0 bytes failed",
    "Could not read repetition level, even though metadata states there are $0 values remaining in data page. file=$1",
    "File '$0' has an incompatible Parquet schema for column '$1'. Column type: $2, Parquet schema:\n$3",
    "Failed to allocate $0 bytes for collection '$1'.\nCurrent buffer size: $2 num tuples: $3.",
    "Temporary device for directory $0 is blacklisted from a previous error and cannot be used.",
    "Temporary file $0 is blacklisted from a previous error and cannot be expanded.",
    "RPC client failed to connect: $0",
    "Metadata for file '$0' appears stale. Try running \"refresh $1\" to reload the file metadata.",
    "File '$0' has an invalid Parquet version number: $1.\nPlease check that it is a valid Parquet file. This error can also occur due to stale metadata. If you believe this is a valid Parquet file, try running \"refresh $2\".",
    "Tried to read $0 bytes but could only read $1 bytes. This may indicate data file corruption. (file $2, byte offset: $3)",
    "Invalid read of $0 bytes. This may indicate data file corruption. (file $1, byte offset: $2)",
    "File '$0' has an invalid version header: $1\nMake sure the file is an Avro data file.",
    "$0's allocations exceeded memory limits.",
    "No longer in use.",
    "For better performance, snappy-, gzip-, and bzip-compressed files should not be split into multiple HDFS blocks. file=$0 offset $1",
    "$0 Data error, likely data corrupted in this block.",
    "$0 Decompressor error at $1, code=$2",
    "Decompression failed to make progress, but end of input is not reached. File appears corrupted. file=$0",
    "Unexpected end of compressed file. File may be truncated. file=$0",
    "Sender$0 timed out waiting for receiver fragment instance: $1, dest node: $2",
    "Kudu type $0 is not available in Impala.",
    "Impala type $0 is not available in Kudu.",
    "Not in use.",
    "Kudu features are disabled by the startup flag --disable_kudu.",
    "Cannot perform hash join at node with id $0. Repartitioning did not reduce the size of a spilled partition. Repartitioning level $1. Number of rows $2:\n$3\n$4",
    "Not in use.",
    "File '$0' is corrupt: truncated data block at offset $1",
    "File '$0' is corrupt: invalid union value $1 at offset $2",
    "File '$0' is corrupt: invalid boolean value $1 at offset $2",
    "File '$0' is corrupt: invalid length $1 at offset $2",
    "File '$0' is corrupt: invalid encoded integer at offset $1",
    "File '$0' is corrupt: invalid record count $1 at offset $2",
    "File '$0' is corrupt: invalid compressed block size $1 at offset $2",
    "File '$0' is corrupt: invalid metadata count $1 at offset $2",
    "File '$0' could not be read: string $1 was longer than supported limit of $2 bytes at offset $3",
    "File '$0' is corrupt: error decoding value of type $1 at offset $2",
    "File '$0' is corrupt: error reading dictionary for data of type $1: $2",
    "Length of column is $0 which exceeds maximum supported length of 2147483647 bytes.",
    "Scratch space limit of $0 bytes exceeded for query while spilling data to disk on backend $1.",
    "Unexpected error allocating $0 byte buffer: $1",
    "File '$0' is corrupt: metadata indicates a zero row count but there is at least one non-empty row group.",
    "Cannot schedule query: no registered backends available.",
    "Key already present in Kudu table '$0'.",
    "Not found in Kudu table '$0': $1",
    "Error in Kudu table '$0': $1",
    "Column '$0': unsupported Avro type '$1'",
    "Column '$0': invalid Avro decimal type with precision = '$1' scale = '$2'",
    "Row with null value violates nullability constraint on table '$0'.",
    "Parquet file '$0' column '$1' contains an out of range timestamp. The valid date range is 1400-01-01..9999-12-31.",
    "Could not create files in any configured scratch directories (--scratch_dirs=$0) on backend '$1'. $2 of scratch is currently in use by this Impala Daemon ($3 by this query). See logs for previous errors that may have prevented creating or writing scratch files. The following directories were at capacity: $4",
    "Error reading $0 bytes from scratch file '$1' on backend $2 at offset $3: could only read $4 bytes",
    "Kudu table '$0' column '$1' contains an out of range timestamp. The valid date range is 1400-01-01..9999-12-31.",
    "Row of size $0 could not be materialized by $1. Increase the max_row_size query option (currently $2) to process larger rows.",
    "Failed to verify generated IR function $0, see log for more details.",
    "Failed to get minimum memory reservation of $0 on daemon $1:$2 for query $3 due to following error: $4Memory is likely oversubscribed. Reducing query concurrency or configuring admission control may help avoid this error.",
    "Rejected query from pool $0: $1",
    "Admission for query exceeded timeout $0ms in pool $1. Queued reason: $2 Additional Details: $3",
    "Failed to create thread $0 in category $1: $2",
    "Disk I/O error on $0: $1",
    "DataStreamRecvr for fragment=$0, node=$1 is closed already",
    "Kerberos principal should be of the form: <service>/<hostname>@<realm> - got: $0",
    "The input size is too large for LZ4 compression: $0",
    "InitAuth() called multiple times with different names. Was called with $0. Now using $1.",
    "Can not read Parquet file $0 with deprecated BIT_PACKED encoding for rep or def levels. Support was removed in Impala 3.0 - see IMPALA-6077.",
    "Row batch cannot be serialized: size of $0 bytes exceeds supported limit of $1",
    "The library $0 last modified time $1 does not match the expected last modified time $2. Run 'refresh functions <db name>'.",
    "Error reading $0 bytes from scratch file '$1' on backend $2 at offset $3: verification of read data failed.",
    "Cancelled in $0",
    "Server is being shut down: $0.",
    "Parquet file '$0' column '$1' contains a timestamp with invalid time of day. The time of day should be 0 <= and < 24 hour (in nanoseconds).",
    "File '$0' is corrupt: error decoding BOOLEAN value with encoding $1 at offset $2",
    "Failed to submit $0 to thread pool after waiting $1 seconds",
    "$0 failed to finish before the $1 second timeout",
    "Failed due to unreachable impalad(s): $0",
    "Session expired due to inactivity",
    "Query $0 expired due to client inactivity (timeout is $1)",
    "Query $0 expired due to execution time limit of $1",
    "Query $0 terminated due to CPU limit of $1",
    "Query $0 terminated due to scan bytes limit of $1",
    "Query $0 terminated due to rows produced limit of $1. Unset or increase NUM_ROWS_PRODUCED_LIMIT query option to produce more rows.",
    "Expression rewrite rejected due to result size ($0) exceeding the limit ($1).",
    "Query $0 cancelled due to unresponsive backend: $1 has not sent a report in $2ms (max allowed lag is $3ms)",
    "Parquet file '$0' column '$1' contains an out of range date. The valid date range is 0001-01-01..9999-12-31.",
    "Session closed because it has no active connections",
    "The user authorized on the connection '$0' does not match the session username '$1'",
    "$0 failed with error: $1",
    "LZ4Block: Decompressed size is not correct.",
    "LZ4Block: Invalid input length.",
    "LZ4Block: Invalid compressed length.  Data is likely corrupt.",
    "LZ4: LZ4_decompress_safe failed",
    "LZ4: LZ4_compress_default failed",
    "Statement length of $0 bytes exceeds the maximum statement length ($1 bytes)",
    "Avro file '$0' is corrupt: out of range date value $1 at offset $2. The valid date range is -719162..2932896 (0001-01-01..9999-12-31).",
    "ORC file '$0' column '$1' contains an out of range timestamp. The valid date range is 1400-01-01..9999-12-31.",
    "ORC file '$0' column '$1' contains an out of range date. The valid date range is 0001-01-01..9999-12-31.",
    "File '$0' has an incompatible ORC schema for column '$1', Column type: $2, ORC schema: $3",
    "Root of the $0 type returned by the ORC lib is not STRUCT: $1. Either there are bugs in the ORC lib or ORC file '$2' is corrupt.",
    "Unable to perform Null-Aware Anti-Join. Could not get enough reservation to fit all rows with NULLs from the build side in memory. Memory required for $0 rows was $1. $2/$3 of the join's reservation was available for the rows.",
    "Invalid or unknown query handle: $0.",
    "Query $0 terminated due to join rows produced exceeds the limit of $1 at node with id $2. Unset or increase JOIN_ROWS_PRODUCED_LIMIT query option to produce more rows.",
    "Query execution failure caused by local disk IO fatal error on backend: $0.",
    "Error parsing JWKS: $0.",
    "Error verifying JWT Token: $0.",
    "Couldn't skip rows in column '$0' in file '$1'.",
    "Failed to parse query option '$0': $1",
    "Client has incompatible protocol version V$0 conflicting with catalogd's version V$1",
    "Subscriber '$0' has incompatible protocol version V$1 conflicting with statestored's version V$2",
    "Error in JDBC table configuration: $0.",
    "Inconsistent tuple cache found: $0.",
    "Error verifying OAuth Token: $0.",
]
